# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from blazar_dashboard import api
from blazar_dashboard.content.leases import forms as project_forms
from blazar_dashboard.content.leases import tables as project_tables
from blazar_dashboard.content.leases import tabs as project_tabs


class IndexView(tables.DataTableView):
    table_class = project_tables.LeasesTable
    template_name = 'project/leases/index.html'

    def get_data(self):
        try:
            leases = api.client.lease_list(self.request)
        except Exception:
            leases = []
            msg = _('Unable to retrieve lease information.')
            exceptions.handle(self.request, msg)
        return leases


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.LeaseDetailTabs
    template_name = 'project/leases/detail.html'


class CreateView(forms.ModalFormView):
    form_class = project_forms.CreateForm
    template_name = 'project/leases/create.html'
    success_url = reverse_lazy('horizon:project:leases:index')
    modal_id = "create_lease_modal"
    modal_header = _("Create Lease")
    submit_label = _("Create Lease")
    submit_url = reverse_lazy('horizon:project:leases:create')


class UpdateView(forms.ModalFormView):
    form_class = project_forms.UpdateForm
    template_name = 'project/leases/update.html'
    success_url = reverse_lazy('horizon:project:leases:index')
    modal_header = _("Update Lease")

    def get_initial(self):
        initial = super(UpdateView, self).get_initial()

        initial['lease'] = self.get_object()
        if initial['lease']:
            initial['lease_id'] = initial['lease'].id
            initial['name'] = initial['lease'].name

        return initial

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['lease'] = self.get_object()
        return context

    @memoized.memoized_method
    def get_object(self):
        lease_id = self.kwargs['lease_id']
        try:
            lease = api.client.lease_get(self.request, lease_id)
        except Exception:
            msg = _("Unable to retrieve lease.")
            redirect = reverse('horizon:project:leases:index')
            exceptions.handle(self.request, msg, redirect=redirect)
        return lease



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows

from blazar_dashboard import api
from blazar_dashboard.content.hosts import forms as project_forms
from blazar_dashboard.content.hosts import tables as project_tables
from blazar_dashboard.content.hosts import tabs as project_tabs
from blazar_dashboard.content.hosts import workflows as project_workflows


class IndexView(tables.DataTableView):
    table_class = project_tables.HostsTable
    template_name = 'admin/hosts/index.html'

    def get_data(self):
        try:
            hosts = api.client.host_list(self.request)
        except Exception:
            hosts = []
            msg = _('Unable to retrieve host information.')
            exceptions.handle(self.request, msg)
        return hosts


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.HostDetailTabs
    template_name = 'admin/hosts/detail.html'


class CreateView(workflows.WorkflowView):
    workflow_class = project_workflows.CreateHostsWorkflow
    template_name = 'admin/hosts/create.html'
    page_title = _("Create Hosts")


class UpdateView(forms.ModalFormView):
    form_class = project_forms.UpdateForm
    template_name = 'admin/hosts/update.html'
    success_url = reverse_lazy('horizon:admin:hosts:index')
    modal_header = _("Update Host")

    def get_initial(self):
        initial = super(UpdateView, self).get_initial()

        initial['host'] = self.get_object()
        if initial['host']:
            initial['host_id'] = initial['host'].id
            initial['name'] = initial['host'].hypervisor_hostname

        return initial

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['host'] = self.get_object()
        return context

    @memoized.memoized_method
    def get_object(self):
        host_id = self.kwargs['host_id']
        try:
            host = api.client.host_get(self.request, host_id)
        except Exception:
            msg = _("Unable to retrieve host.")
            redirect = reverse_lazy('horizon:admin:hosts:index')
            exceptions.handle(self.request, msg, redirect=redirect)
        return host



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.test_data import utils

from blazar_dashboard import api


lease_sample1 = {
    'status': None,
    'user_id': None,
    'name': 'lease-1',
    'end_date': '2030-06-30T18:00:00.000000',
    'reservations': [
        {
            'status': 'pending',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'min': 1,
            'max': 1,
            'hypervisor_properties': '',
            'resource_id': '3850a831-8c08-49c4-b703-d804284a6baf',
            'resource_properties': '[">=", "$vcpus", "2"]',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'id': '087bc740-6d2d-410b-9d47-c7b2b55a9d36',
            'resource_type': 'physical:host'
        }
    ],
    'created_at': '2017-06-27 15:00:00',
    'updated_at': None,
    'events': [
        {
            'status': 'UNDONE',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'event_type': 'start_lease',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'time': '2017-06-27T18:00:00.000000',
            'id': '188a8584-f832-4df9-9a4a-51e6364420ff'
        },
        {
            'status': 'UNDONE',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'event_type': 'end_lease',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'time': '2030-06-30T18:00:00.000000',
            'id': '277d6436-dfcb-4eae-ae5e-ac7fa9c2fd56'
        },
        {
            'status': 'UNDONE',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'event_type': 'before_end_lease',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'time': '2030-06-28T18:00:00.000000',
            'id': 'f583af71-ca21-4b66-87de-52211d118029'
        }
    ],
    'id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
    'action': None,
    'project_id': 'aa45f56901ef45ee95e3d211097c0ea3',
    'status_reason': None,
    'start_date': '2017-06-27T18:00:00.000000',
    'trust_id': 'b442a580b9504ababf305bf2b4c49512'
}

lease_sample2 = {
    'status': None,
    'user_id': None,
    'name': 'lease-2',
    'end_date': '2030-06-30T18:00:00.000000',
    'reservations': [
        {
            'status': 'pending',
            'lease_id': '9bcfff36-872e-4f47-9abe-9a58a4f22038',
            'min': 1,
            'max': 1,
            'hypervisor_properties': '',
            'resource_id': '369c83cb-d3de-4e15-9e15-b74625cf9ee5',
            'resource_properties': '[">=", "$vcpus", "2"]',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'id': '1b05370e-d92a-452d-80db-89842666b604',
            'resource_type': 'physical:host'
        }
    ],
    'created_at': '2017-06-27 15:00:00',
    'updated_at': None,
    'events': [
        {
            'status': 'UNDONE',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'event_type': 'start_lease',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'time': '2017-06-27T18:00:00.000000',
            'id': '0d81cdd7-9390-4d19-8acf-746bc8f0167d'
        },
        {
            'status': 'UNDONE',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'event_type': 'end_lease',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'time': '2030-06-30T18:00:00.000000',
            'id': 'b2ac8924-b6d1-46fe-b1a9-43d9d5c683cf'
        },
        {
            'status': 'UNDONE',
            'lease_id': '6ee55c78-ac52-41a6-99af-2d2d73bcc466',
            'event_type': 'before_end_lease',
            'created_at': '2017-06-27 15:00:00',
            'updated_at': None,
            'time': '2030-06-28T18:00:00.000000',
            'id': 'ba97b406-e721-47fe-9097-8ce6569f15d3'
        }
    ],
    'id': 'ef32abe8-a1f7-4c2f-b5f2-941428848230',
    'action': None,
    'project_id': 'aa45f56901ef45ee95e3d211097c0ea3',
    'status_reason': None,
    'start_date': '2017-06-27T18:00:00.000000',
    'trust_id': 'b442a580b9504ababf305bf2b4c49512'
}

host_sample1 = {
    "status": None,
    "hypervisor_type": "QEMU",
    "created_at": "2017-10-01 12:00:00",
    "updated_at": None,
    "hypervisor_hostname": "compute-1",
    "memory_mb": 4096,
    "cpu_info": "{'dummy': 'true'}",
    "vcpus": 1,
    "service_name": "blazar",
    "hypervisor_version": 2005000,
    "local_gb": 128,
    "id": "1",
    "trust_id": "dummy",
    "ex1": "dummy"
}

host_sample2 = {
    "status": None,
    "hypervisor_type": "QEMU",
    "created_at": "2017-10-01 12:00:00",
    "updated_at": None,
    "hypervisor_hostname": "compute-2",
    "memory_mb": 4096,
    "cpu_info": "{'dummy': 'true'}",
    "vcpus": 1,
    "service_name": "blazar",
    "hypervisor_version": 2005000,
    "local_gb": 128,
    "id": "2",
    "trust_id": "dummy",
    "ex2": "dummy"
}


class DummyNovaHost(object):
    def __init__(self, host_name, service):
        self.host_name = host_name
        self.service = service

novahost_sample1 = DummyNovaHost('compute-1', 'compute')
novahost_sample2 = DummyNovaHost('compute-2', 'compute')


def data(TEST):
    TEST.leases = utils.TestDataContainer()

    TEST.leases.add(api.client.Lease(lease_sample1))
    TEST.leases.add(api.client.Lease(lease_sample2))

    TEST.hosts = utils.TestDataContainer()

    TEST.hosts.add(api.client.Host(host_sample1))
    TEST.hosts.add(api.client.Host(host_sample2))

    TEST.novahosts = utils.TestDataContainer()

    TEST.novahosts.add(novahost_sample1)
    TEST.novahosts.add(novahost_sample2)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django import http
from mox3.mox import IsA
from openstack_dashboard import api

from blazar_dashboard import api as blazar_api
from blazar_dashboard.test import helpers as test

import logging
LOG = logging.getLogger(__name__)

INDEX_TEMPLATE = 'admin/hosts/index.html'
INDEX_URL = reverse('horizon:admin:hosts:index')
DETAIL_TEMPLATE = 'admin/hosts/detail.html'
DETAIL_URL_BASE = 'horizon:admin:hosts:detail'
CREATE_URL = reverse('horizon:admin:hosts:create')
CREATE_TEMPLATE = 'admin/hosts/create.html'
UPDATE_URL_BASE = 'horizon:admin:hosts:update'
UPDATE_TEMPLATE = 'admin/hosts/update.html'


class HostsTests(test.BaseAdminViewTests):
    @test.create_stubs({blazar_api.client: ('host_list',)})
    def test_index(self):
        hosts = self.hosts.list()
        blazar_api.client.host_list(IsA(http.HttpRequest)).AndReturn(hosts)
        self.mox.ReplayAll()

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.assertNoMessages(res)
        self.assertContains(res, 'compute-1')
        self.assertContains(res, 'compute-2')

    @test.create_stubs({blazar_api.client: ('host_list',)})
    def test_index_no_hosts(self):
        blazar_api.client.host_list(IsA(http.HttpRequest)).AndReturn(())
        self.mox.ReplayAll()

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.assertNoMessages(res)
        self.assertContains(res, 'No items to display')

    @test.create_stubs({blazar_api.client: ('host_list',)})
    def test_index_error(self):
        blazar_api.client.host_list(
            IsA(http.HttpRequest)
        ).AndRaise(self.exceptions.blazar)
        self.mox.ReplayAll()

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.assertMessageCount(res, error=1)

    @test.create_stubs({blazar_api.client: ('host_get',)})
    def test_host_detail(self):
        host = self.hosts.get(hypervisor_hostname='compute-1')
        blazar_api.client.host_get(IsA(http.HttpRequest),
                                   host['id']).AndReturn(host)
        self.mox.ReplayAll()

        res = self.client.get(reverse(DETAIL_URL_BASE, args=[host['id']]))
        self.assertTemplateUsed(res, DETAIL_TEMPLATE)
        self.assertContains(res, 'compute-1')
        self.assertContains(res, 'ex1')

    @test.create_stubs({blazar_api.client: ('host_get',)})
    def test_host_detail_error(self):
        blazar_api.client.host_get(IsA(http.HttpRequest),
                                   'invalid').AndRaise(self.exceptions.blazar)
        self.mox.ReplayAll()

        res = self.client.get(reverse(DETAIL_URL_BASE, args=['invalid']))
        self.assertTemplateNotUsed(res, DETAIL_TEMPLATE)
        self.assertMessageCount(error=1)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @test.create_stubs({blazar_api.client: ('host_list', 'host_create',),
                        api.nova: ('host_list',)})
    def test_create_hosts(self):
        blazar_api.client.host_list(IsA(http.HttpRequest)
                                    ).AndReturn([])
        api.nova.host_list(IsA(http.HttpRequest)
                           ).AndReturn(self.novahosts.list())
        host_names = [h.host_name for h in self.novahosts.list()]
        for host_name in host_names:
            blazar_api.client.host_create(
                IsA(http.HttpRequest),
                name=host_name,
            ).AndReturn([])
        self.mox.ReplayAll()
        form_data = {
            'select_hosts_role_member': host_names
        }

        res = self.client.post(CREATE_URL, form_data)
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=(len(host_names) + 1))
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @test.create_stubs({blazar_api.client: ('host_list', 'host_create',),
                        api.nova: ('host_list',)})
    def test_create_hosts_with_extra_caps(self):
        blazar_api.client.host_list(IsA(http.HttpRequest)
                                    ).AndReturn([])
        api.nova.host_list(IsA(http.HttpRequest)
                           ).AndReturn(self.novahosts.list())
        host_names = [h.host_name for h in self.novahosts.list()]
        for host_name in host_names:
            blazar_api.client.host_create(
                IsA(http.HttpRequest),
                name=host_name,
                extracap="strong"
            ).AndReturn([])
        self.mox.ReplayAll()
        form_data = {
            'select_hosts_role_member': host_names,
            'extra_caps': '{"extracap": "strong"}'
        }

        res = self.client.post(CREATE_URL, form_data)
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=(len(host_names) + 1))
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @test.create_stubs({blazar_api.client: ('host_get', 'host_update')})
    def test_update_host(self):
        host = self.hosts.get(hypervisor_hostname='compute-1')
        blazar_api.client.host_get(
            IsA(http.HttpRequest),
            host['id']
        ).AndReturn(host)
        blazar_api.client.host_update(
            IsA(http.HttpRequest),
            host_id=host['id'],
            values={"key": "updated"}
        )
        form_data = {
            'host_id': host['id'],
            'values': '{"key": "updated"}'
        }
        self.mox.ReplayAll()

        res = self.client.post(reverse(UPDATE_URL_BASE, args=[host['id']]),
                               form_data)
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=1)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @test.create_stubs({blazar_api.client: ('host_get', 'host_update')})
    def test_update_host_error(self):
        host = self.hosts.get(hypervisor_hostname='compute-1')
        blazar_api.client.host_get(
            IsA(http.HttpRequest),
            host['id']
        ).AndReturn(host)
        blazar_api.client.host_update(
            IsA(http.HttpRequest),
            host_id=host['id'],
            values={"key": "updated"}
        ).AndRaise(self.exceptions.blazar)
        form_data = {
            'host_id': host['id'],
            'values': '{"key": "updated"}'
        }
        self.mox.ReplayAll()

        res = self.client.post(reverse(UPDATE_URL_BASE, args=[host['id']]),
                               form_data)
        self.assertNoFormErrors(res)
        self.assertContains(res, 'An error occurred while updating')

    @test.create_stubs({blazar_api.client: ('host_list', 'host_delete')})
    def test_delete_host(self):
        hosts = self.hosts.list()
        host = self.hosts.get(hypervisor_hostname='compute-1')
        blazar_api.client.host_list(IsA(http.HttpRequest)).AndReturn(hosts)
        blazar_api.client.host_delete(IsA(http.HttpRequest), host['id'])
        self.mox.ReplayAll()

        action = 'hosts__delete__%s' % host['id']
        form_data = {'action': action}
        res = self.client.post(INDEX_URL, form_data)
        self.assertMessageCount(success=1)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @test.create_stubs({blazar_api.client: ('host_list', 'host_delete')})
    def test_delete_host_error(self):
        hosts = self.hosts.list()
        host = self.hosts.get(hypervisor_hostname='compute-1')
        blazar_api.client.host_list(IsA(http.HttpRequest)).AndReturn(hosts)
        blazar_api.client.host_delete(
            IsA(http.HttpRequest),
            host['id']).AndRaise(self.exceptions.blazar)
        self.mox.ReplayAll()

        action = 'hosts__delete__%s' % host['id']
        form_data = {'action': action}
        res = self.client.post(INDEX_URL, form_data)
        self.assertMessageCount(error=1)
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import tabs

from blazar_dashboard.api import client


class OverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = "admin/hosts/_detail_overview.html"

    def get_context_data(self, request):
        host_id = self.tab_group.kwargs['host_id']
        try:
            host = client.host_get(self.request, host_id)
        except Exception:
            redirect = reverse('horizon:admin:hosts:index')
            msg = _('Unable to retrieve host details.')
            exceptions.handle(request, msg, redirect=redirect)

        return {'host': host}


class HostDetailTabs(tabs.TabGroup):
    slug = "host_details"
    tabs = (OverviewTab,)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

import logging

from horizon import exceptions
from horizon.utils.memoized import memoized
from openstack_dashboard.api import base

from blazarclient import client as blazar_client


LOG = logging.getLogger(__name__)


class Lease(base.APIDictWrapper):
    """Represents one Blazar lease."""
    ACTIONS = (CREATE, DELETE, UPDATE, START, STOP
               ) = ('CREATE', 'DELETE', 'UPDATE', 'START', 'STOP')

    STATUSES = (IN_PROGRESS, FAILED, COMPLETE
                ) = ('IN_PROGRESS', 'FAILED', 'COMPLETE')

    _attrs = ['id', 'name', 'start_date', 'end_date', 'user_id', 'project_id',
              'before_end_date', 'action', 'status', 'status_reason']

    def __init__(self, apiresource):
        super(Lease, self).__init__(apiresource)


class Host(base.APIDictWrapper):
    """Represents one Blazar host."""

    _attrs = ['id', 'hypervisor_hostname', 'hypervisor_type',
              'hypervisor_version', 'vcpus', 'cpu_info', 'memory_mb',
              'local_gb', 'status', 'created_at', 'updated_at',
              'service_name', 'trust_id']

    def __init__(self, apiresource):
        super(Host, self).__init__(apiresource)

    def cpu_info_dict(self):
        cpu_info_dict = getattr(self, 'cpu_info', '{}')
        if not cpu_info_dict:
            cpu_info_dict = '{}'
        return eval(cpu_info_dict)

    def extra_capabilities(self):
        excaps = {}
        for k, v in self._apidict.items():
            if k not in self._attrs:
                excaps[k] = v
        return excaps


@memoized
def blazarclient(request):
    try:
        api_url = base.url_for(request, 'reservation')
    except exceptions.ServiceCatalogException:
        LOG.debug('No Reservation service is configured.')
        return None

    LOG.debug('blazarclient connection created using the token "%s" and url'
              '"%s"' % (request.user.token.id, api_url))
    return blazar_client.Client(
        blazar_url=api_url,
        auth_token=request.user.token.id)


def lease_list(request):
    """List the leases."""
    leases = blazarclient(request).lease.list()
    return [Lease(l) for l in leases]


def lease_get(request, lease_id):
    """Get a lease."""
    lease = blazarclient(request).lease.get(lease_id)
    return Lease(lease)


def lease_create(request, name, start, end, reservations, events):
    """Create a lease."""
    lease = blazarclient(request).lease.create(
        name, start, end, reservations, events)
    return Lease(lease)


def lease_update(request, lease_id, **kwargs):
    """Update a lease."""
    lease = blazarclient(request).lease.update(lease_id, **kwargs)
    return Lease(lease)


def lease_delete(request, lease_id):
    """Delete a lease."""
    blazarclient(request).lease.delete(lease_id)


def host_list(request):
    """List hosts."""
    hosts = blazarclient(request).host.list()
    return [Host(h) for h in hosts]


def host_get(request, host_id):
    """Get a host."""
    host = blazarclient(request).host.get(host_id)
    return Host(host)


def host_create(request, name, **kwargs):
    """Create a host."""
    host = blazarclient(request).host.create(name, **kwargs)
    return Host(host)


def host_update(request, host_id, values):
    """Update a host."""
    host = blazarclient(request).host.update(host_id, values)
    return Host(host)


def host_delete(request, host_id):
    """Delete a host."""
    blazarclient(request).host.delete(host_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import workflows
from openstack_dashboard import api

from blazar_dashboard import api as blazar_api

LOG = logging.getLogger(__name__)


class SelectHostsAction(workflows.MembershipAction):
    def __init__(self, request, *args, **kwargs):
        super(SelectHostsAction, self).__init__(request, *args, **kwargs)
        err_msg = _('Unable to get the available hosts')

        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)

        try:
            nova_hosts = api.nova.host_list(request)
            blazar_hosts = blazar_api.client.host_list(request)
        except Exception:
            exceptions.handle(request, err_msg)

        nova_hostnames = []
        for host in nova_hosts:
            if (host.host_name not in nova_hostnames
                    and host.service == u'compute'):
                nova_hostnames.append(host.host_name)

        blazar_hostnames = []
        for host in blazar_hosts:
            if host.hypervisor_hostname not in blazar_hostnames:
                blazar_hostnames.append(host.hypervisor_hostname)

        host_names = list(set(nova_hostnames) - set(blazar_hostnames))
        host_names.sort()

        self.fields[field_name].choices = \
            [(host_name, host_name) for host_name in host_names]

        self.fields[field_name].initial = None

    class Meta(object):
        name = _("Select Hosts")
        slug = "select_hosts"


class AddExtraCapsAction(workflows.Action):
    extra_caps = forms.CharField(
        label=_("Extra Capabilities"),
        required=False,
        help_text=_('Enter extra capabilities of hosts in JSON'),
        widget=forms.Textarea(
            attrs={'rows': 5}),
        max_length=511)

    class Meta(object):
        name = _("Extra Capabilities")
        slug = "add_extra_caps"

    def clean(self):
        cleaned_data = super(AddExtraCapsAction, self).clean()
        extra_caps = cleaned_data.get('extra_caps')

        if extra_caps:
            try:
                extra_caps = eval(extra_caps)
                cleaned_data['extra_caps'] = extra_caps
            except (SyntaxError, NameError):
                raise forms.ValidationError(
                    _('Extra capabilities must written in JSON')
                )

        return cleaned_data


class SelectHostsStep(workflows.UpdateMembersStep):
    action_class = SelectHostsAction
    help_text = _("Select hosts to create")
    available_list_title = _("All available hosts")
    members_list_title = _("Selected hosts")
    no_available_text = _("No host found.")
    no_members_text = _("No host selected.")
    show_roles = False
    contributes = ("names",)

    def contribute(self, data, context):
        if data:
            member_field_name = self.get_member_field_name('member')
            context['names'] = data.get(member_field_name, [])
        return context


class AddExtraCapsStep(workflows.Step):
    action_class = AddExtraCapsAction
    help_text = _("Add extra capabilities")
    show_roles = False
    contributes = ("extra_caps",)

    def contribute(self, data, context):
        context['extra_caps'] = data.get('extra_caps')
        return context


class CreateHostsWorkflow(workflows.Workflow):
    slug = "create_hosts"
    name = _("Create Hosts")
    finalize_button_name = _("Create Hosts")
    success_url = 'horizon:admin:hosts:index'
    default_steps = (SelectHostsStep, AddExtraCapsStep)

    def handle(self, request, context):
        try:
            for name in context['names']:
                if context['extra_caps']:
                    blazar_api.client.host_create(request, name=name,
                                                  **context['extra_caps'])
                else:
                    blazar_api.client.host_create(request, name=name)
                messages.success(request, _('Host %s was successfully '
                                            'created.') % name)
        except Exception:
            exceptions.handle(request, _('Unable to create host.'))
            return False

        return True
